#include "opencv2/opencv.hpp"

using namespace cv;
using namespace std;

Mat getHistogramImage(MatND &hist);

int main(){
    
  Mat src;

  src = imread( "lenna.png");
  imshow("Lenna", src);

  // Separa os canais da imagem 
  vector<Mat> bgr_planes;
  split( src, bgr_planes );

  // Determina a quantidade de bins
  int number_bins = 256;

  // Declara cada histograma. Um para cada canal
  Mat b_hist, g_hist, r_hist;

  //calcula histograma para cada canal
  calcHist( &bgr_planes[0], 1, 0, Mat(), b_hist, 1, &number_bins, NULL );
  calcHist( &bgr_planes[1], 1, 0, Mat(), g_hist, 1, &number_bins, NULL );
  calcHist( &bgr_planes[2], 1, 0, Mat(), r_hist, 1, &number_bins, NULL );

  //normaliza cada histograma
  normalize(b_hist, b_hist, 1);
  normalize(g_hist, g_hist, 1);
  normalize(r_hist, r_hist, 1);
  
  Mat histBlue = getHistogramImage(b_hist);
  Mat histGreen = getHistogramImage(g_hist);
  Mat histRed = getHistogramImage(r_hist);
  imshow("Blue", histBlue );
  imshow("Green", histGreen );
  imshow("Red", histRed );
  cvWaitKey(0);
    
  return 0;
}

Mat getHistogramImage(MatND &hist){
    
    double maxVal=0;
    double minVal=0;
    cv::minMaxLoc(hist, &minVal, &maxVal, 0, 0);
    
    //cria imagem onde será desenhado o histograma
    Mat histImg(256, 256,
                    CV_8U,cv::Scalar(255));
    
    //determina o maior H como sendo 90% da imagem
    int hpt = static_cast<int>(0.9*256);
    
    //para cada bin em hist, desenha uma linha vertical
    for( int h = 0; h < 256; h++ ) {
       float binVal = hist.at<float>(h);
       int intensity = static_cast<int>(binVal*hpt/maxVal);
       
       //desenha uma linha entre dois pontos
       line(histImg,Point(h,256),Point(h,256-intensity),Scalar::all(0));
    }
  return histImg;
}